import labscript_utils.h5_lock
import h5py
import os
import threading
from . import uhd_lib, send_tones
from blacs.tab_base_classes import Worker
from zprocess import rich_print

BLUE = '#66D9EF'
PURPLE = '#AE81FF'
GREEN = '#A6E22E'
GREY = '#75715E' 

class EttusSDRWorker(Worker):
    def init(self):
        self.usrp = uhd_lib.initialize_sdr(self.tx_frequency, self.sampling_rate)
    def program_manual(self, values):
        return {}

    def transition_to_buffered(self, device_name, h5_file, initial_values, fresh):
        rich_print(f"====== new shot: {os.path.basename(h5_file)} ======", color=GREEN)
        with h5py.File(h5_file,'r') as f:
            group = f[f"devices/{device_name}"]
            tx_frequency = group.attrs["tx_frequency"]
            sampling_rate = group.attrs["sampling_rate"]
            duration = group.attrs["duration"]
            tones = group["data"][:]

        self.stream = threading.Thread(
            target = send_tones.main,
            args = (self.usrp, duration, sampling_rate, tones["frequencies"], tones["amplitudes"], tones["phases"])
        )
        self.stream.daemon = True
        self.stream.start()
        plural = "s" if duration != 1 else "" 
        rich_print(f"[Started streaming for {duration} second{plural}]", color=GREY)
        return {}

    def transition_to_manual(self):
        # while self.stream.is_alive():
        #     rich_print("[Stream not yet finished]", color=PURPLE)
        #     time.sleep(1)
        # rich_print("[Thread stopped]", color=BLUE)
        rich_print("[Finished]", color=GREY)
        return True

    def shutdown(self):
        return

    def abort_buffered(self):
        return self.transition_to_manual()

    def abort_transition_to_buffered(self):
        return True
